/** Includes */
#include "lpc_types.h"
#include "lpc13xx_uart.h"

/** Define global variables */
#define BUFSIZE 16
uint8_t RxData[BUFSIZE];
int iRx;

/** Define private functions */
void UART_IRQHandler(void);
void init_UART(uint32_t baudrate);
void Transmit_UART();

/**
 * @brief Receiver interrupt
 * @param[in] NONE
 * @return NONE
 */
void ReceiverInterrupt()
{
	init_UART(115200);
	while(1)
	{
		Transmit_UART();
	}
}
/**
 * @brief ISR UART
 * @param[in] NONE
 * @return NONE
 */
void UART_IRQHandler(void)
{
    uint8_t IIRValue, LSRValue;
    uint8_t Dummy = Dummy;

    IIRValue = LPC_UART->IIR;
    IIRValue >>= 1;            /* skip pending bit in IIR */
    IIRValue &= 0x0E;        /* check bit 1~3, interrupt identification */
    IIRValue <<= 1;                                                                        
    if (IIRValue == UART_IIR_INTID_RLS)        /* Receive Line Status */
    {
        LSRValue = LPC_UART->LSR;
        /* Receive Line Status */
        if (LSRValue & (UART_LSR_OE | UART_LSR_PE | UART_LSR_FE | UART_LSR_RXFE | UART_LSR_BI))
        {
            /* There are errors or break interrupt */
            /* Read LSR will clear the interrupt */
            //UARTStatus = LSRValue;
            Dummy = LPC_UART->RBR;    /* Dummy read on RX to clear interrupt, then bail out */
            return;
        }
        if (LSRValue & UART_LSR_RDR)    /* Receive Data Ready */
        {
            /* If no error on RLS, normal ready, save into the data buffer. */
            /* Note: read RBR will clear the interrupt */
            RxData[iRx++] = LPC_UART->RBR;
            if (iRx == BUFSIZE)
            {
                iRx = 0;        /* buffer overflow */
            }
        }
     }
     else if (IIRValue == UART_IIR_INTID_RDA)    /* Receive Data Available */
     {
         /* Receive Data Available */
         RxData[iRx++] = LPC_UART->RBR;
         if (iRx >= BUFSIZE)
         {
             iRx = 0;        /* buffer overflow */
         }
     }
}
/**
 * @brief Init UART
 * @param[in] U32 : Baud Rate
 * @return NONE
 */
void init_UART(uint32_t baudrate)
{
	uint32_t Fdiv;
	uint32_t regVal;

	NVIC_DisableIRQ(UART_IRQn);
	LPC_IOCON->PIO1_6 &= ~0x07;    /*  UART I/O config */
	LPC_IOCON->PIO1_6 |= 0x01;     /* UART RXD */
	LPC_IOCON->PIO1_7 &= ~0x07;
	LPC_IOCON->PIO1_7 |= 0x01;     /* UART TXD */
	/* Enable UART clock */
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
	LPC_SYSCON->UARTCLKDIV = 0x1;     /* divided by 1 */

	LPC_UART->LCR = 0x83;             /* 8 bits, no Parity, 1 Stop bit */
	regVal = LPC_SYSCON->UARTCLKDIV;
	Fdiv = (((72000000*LPC_SYSCON->SYSAHBCLKDIV)/regVal)/16)/baudrate ;    /*baud rate */

	LPC_UART->DLM = Fdiv / 256;
	LPC_UART->DLL = Fdiv % 256;
	LPC_UART->LCR = 0x03;        /* DLAB = 0 */
	LPC_UART->FCR = 0x07;        /* Enable and reset TX and RX FIFO. */

    /* Read to clear the line status. */
	regVal = LPC_UART->LSR;
	/* Ensure a clean start, no data in either TX or RX FIFO. */
	// CodeRed - added parentheses around comparison in operand of &
	while (( LPC_UART->LSR & (UART_LSR_THRE|UART_LSR_TEMT)) != (UART_LSR_THRE|UART_LSR_TEMT) );
	while ( LPC_UART->LSR & UART_LSR_RDR )
	{
		regVal = LPC_UART->RBR;    /* Dump data from RX FIFO */
	}
	/* Enable the UART Interrupt */
	LPC_UART->IER = UART_IER_RBRINT_EN | UART_IER_RLSINT_EN;
	//NVIC_SetPriority(UART_IRQn, 10);	/*Set a Priority for UART*/
	NVIC_EnableIRQ(UART_IRQn);		/*Enable UART Interrupt*/
}
/**
 * @brief Transmit 
 * @param[in] NONE
 * @return NONE
 */
void Transmit_UART()
{
	if (iRx)
    {
    	UART_Send(LPC_UART, RxData, iRx, NONE_BLOCKING);
    	iRx = 0;
    }
}